from selenium import webdriver


class SeleniumStorage:

    def __init__(self, driver: webdriver):
        self.driver = driver

    def __len__(self):
        return self.driver.execute_script("return window.localStorage.length;")

    def items(self):
        return self.driver.execute_script( \
            "var ls = window.localStorage, items = {}; " \
            "for (var i = 0, k; i < ls.length; ++i) " \
            "  items[k = ls.key(i)] = ls.getItem(k); " \
            "return items; ")

    def keys(self):
        return self.driver.execute_script( \
            "var ls = window.localStorage, keys = []; " \
            "for (var i = 0; i < ls.length; ++i) " \
            "  keys[i] = ls.key(i); " \
            "return keys; ")

    def get(self, key: str):
        return self.driver.execute_script("return window.localStorage.getItem(arguments[0]);", key)

    def set(self, key: str, value: str):
        self.driver.execute_script("window.localStorage.setItem(arguments[0], arguments[1]);", key, value)

    def has(self, key: str):
        return key in self.keys()

    def remove(self, key: str):
        self.driver.execute_script("window.localStorage.removeItem(arguments[0]);", key)

    def clear(self):
        self.driver.execute_script("window.localStorage.clear();")




import allure
from bqpackage.selenium_base.selenium_actions import SeleniumActions
from bqpackage.selenium_base.selenium_config import SeleniumConfig

# init selenium driver
why_seebo = "(//a[contains(text(),'Why Seebo')])[1]"
the_solution = "(//a[contains(text(),'The Solution')])[1]"
customers = "(//a[contains(text(),'Customers')])[1]"
get_a_demo = "//li[@id='menu-item-64909']/a/div/div/div/span"
name_field = "(//input)[1]"


# init driver and do test
@allure.feature('Fixtures')
def test():
    selenium = SeleniumConfig()

    DEFAULT_LINK = "https://www.seebo.com/"
    selenium.get_url(DEFAULT_LINK)

    selenium.actions.click(why_seebo)
    selenium.tear_down()

test()


"""
This module wrapping selenium utilities like initiate driver, basic actions, proxy server, and more
"""
import os

import allure
from selenium import webdriver
from webdrivermanager import ChromeDriverManager
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.proxy import ProxyType, Proxy
from bqpackage.selenium_base.selenium_actions import SeleniumActions
from bqpackage.selenium_base.selenium_storage import SeleniumStorage


@allure.step
class SeleniumConfig:
    """Default parameters for initiate driver"""

    @allure.step('Selenium driver initilaziation')
    def __init__(self, headless=False, proxy=False, storage=False, network=False):
        # TODO Change static path
        """Creates a new instance of the chrome driver.
           Starts the service and then creates new instance of chrome driver.

        :Args
        - url - set end point
        - headless - gives the option to run the tests in the background
        - proxy - gives the option to record the traffic

        :Returns:
             - the driver object"""

        self.CHROME_DRIVER_LOCATION = \
            os.path.join(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')), 'chromedriver')
        self.INSTALL_DRIVER = False
        self.PROXY_HOST = "127.0.0.1:8080"
        self.DEFAULT_URL = "chrome://settings/cookies"

        chrome_options = Options()
        capabilities = DesiredCapabilities.CHROME.copy()

        if self.INSTALL_DRIVER:
            self.install_the_latest_chrome_driver()

        if proxy:
            proxy_server = Proxy()
            proxy_server.proxy_type = ProxyType.MANUAL
            proxy_server.http_proxy = f'--proxy-server={self.PROXY_HOST}'

            chrome_options.add_argument(proxy_server.http_proxy)
            proxy_server.add_to_capabilities(capabilities)

        if network:
            capabilities = webdriver.DesiredCapabilities.CHROME
            capabilities['goog:loggingPrefs'] = {'performance': 'ALL'}

        if headless:
            chrome_options.add_argument("--headless")

        self.driver = webdriver.Chrome(executable_path=self.CHROME_DRIVER_LOCATION, chrome_options=chrome_options,
                                       desired_capabilities=capabilities)
        print("Started a new driver")

        if storage:
            self.storage = SeleniumStorage(self.driver)

        self.actions = SeleniumActions(self.driver)

    @allure.step
    def install_the_latest_chrome_driver(self):
        self.driver = webdriver.Chrome(ChromeDriverManager().download_and_install())

    @allure.step
    def get_url(self, url):
        """Loads a web page in the current browser session.

        :Args:
        - driver - needs driver instance
        - url - needs end point url for start session

        :Returns:
             - the driver object"""

        try:
            self.driver.set_page_load_timeout(50)
            self.driver.get(url)
            self.driver.implicitly_wait(10)
            self.driver.maximize_window()
        except Exception as e:
            print(f'Failed to get url : {url}, reason : {e}')
            assert False

    @allure.step
    def tear_down(self):
        """close the driver session."""
        self.driver.close()
        print("driver close")

    @allure.step
    def process_browser_log_entry(self, driver):
        """print the log entry"""
        browser_log = driver.get_log('performance')
        for entry in browser_log:
            print(entry)



"""
This module gives the my-sql capabilities
"""

import mysql
from mysql.connector import Error


def get_connection():
    try:
        connection = mysql.connector.connect(host='localhost',
                                             database='sys',
                                             user='root',
                                             password='password')
        if connection.is_connected():
            db_Info = connection.get_server_info()
            print("Connected to MySQL Server version ", db_Info)
            cursor = connection.cursor()
            cursor.execute("select database();")
            record = cursor.fetchone()
            print("You're connected to database: ", record)

    except Error as e:
        print("Error while connecting to MySQL", e)

    return connection, cursor


def close_connection(self):
    if self.is_connected():
        # cursor.close()
        self.close()
        print("MySQL connection is closed")


def get_data(q):
    try:
        data = []
        index = 0
        db_objects = get_connection();
        db_objects[1].execute(q)
        row = db_objects[1].fetchone()

        while row is not None:
            row = db_objects[1].fetchone()
            data.insert(index, row)
            index += 1

    except Error as e:
        print(e)

    finally:
        db_objects[1].close()
        db_objects[0].close()

    return data



"""
This module contains API requests utilities for tests
"""
import re
import allure

@allure.step
def url_validator(url):
    """ standard validation function that check if provided string is valid url"""
    regex = re.compile(
        r'^(?:http|ftp)s?://'  # http:// or https://
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    return re.match(regex, url) is not None



"""
This module gives the option to run any function in parallel
"""

import time
from multiprocessing import Process


def run_parallel(count_of_process, function_target):
    start = time.perf_counter()
    process_index = 0
    process_list = []
    while process_index < count_of_process:
        temp_process = Process(target=function_target)
        process_list.insert(process_index, temp_process)
        process_index += 1

    for process in process_list:
        process.start()

    for process in process_list:
        process.join()

    finish = time.perf_counter()
    print(f'Finished in {round(finish - start, 2)} second(s)')



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='bqpackage',
      version='0.0.6',
      description='Qualitest package automation infrastructure kit tools for GUI and API tests',
      url='https://upload.pypi.org/legacy/',
      author='Yossi & Moshe',
      long_description=long_description,
      long_description_content_type="text/markdown",
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      use_scm_version=True,
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      python_requires='>=3.7',
      )



"""
This module wrapping basic selenium actions
"""
import re

import allure
from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC


@allure.step
class SeleniumActions:

    @allure.step('Selenium actions initilaziation')
    def __init__(self, driver):
        self._list_of_elements = []
        self._web_element_type = "webelement"
        self._temp_element = None
        self._wait_seconds = 0
        self.driver = driver

    @allure.step
    def wait_for_text(self, location=None, text=None):
        try:
            if location:
                WebDriverWait(self.driver, 10).until(
                    EC.text_to_be_present_in_element((By.CSS_SELECTOR, location), text))
                print(f"Succes to find text: {text} in element: {location}")
            else:
                text = self.find_element_by_xpath(f"//*[text()='{text}']").text
                print(f"Succes to find text: {text}")
            return True
        except Exception:
            assert False, f"Failed to find text: {text}"

    @allure.step
    def get_list_of_elements_by_type(self, element_type, element):
        """
        - args -
              element_type: 1 = id, 2 - xpath, 3 - name, 4 - css, 5 - link text"""
        try:
            if element_type == 1:
                self._list_of_elements = self.driver.find_elements(By.ID, element)
            elif element_type == 2:
                self._list_of_elements = self.driver.find_elements(By.XPATH, element)
            elif element_type == 3:
                self._list_of_elements = self.driver.find_elements(By.NAME, element)
            elif element_type == 4:
                self._list_of_elements = self.driver.find_elements(By.CSS_SELECTOR, element)
            elif element_type == 5:
                self._list_of_elements = self.driver.find_elements(By.LINK_TEXT, element)

            return self._list_of_elements

        except Exception as e:
            print(f"Failed to find list of elements: {e}")
        return

    @allure.step
    def click(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.click()
                print(f"success to click on element: {element}")
                return True
        except Exception as e:
            assert False, f"Failed to click on element: {e}"

    @allure.step
    def hover(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                hover = ActionChains(self.driver).move_to_element(self._temp_element)
                hover.perform()
                print(f"success to hover on: {element}")
                return True
        except Exception as e:
            assert False, f"Failed to hover on element ,reason: {e}"

    @allure.step
    def send_keys(self, element=None, text=None, wait=0):
        text = str(text)
        sleep(wait)
        if not element:
            ActionChains(self.driver).send_keys(text).perform()
            print(f"Succeed to send text to screen")
            return True
        self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                self._temp_element.send_keys(text)
                print(f"Succeed to send text to element: {element}")
                return True
        except Exception as e:
            assert False, f"Failed to send text to element: {e}"

    @allure.step
    def get_text(self, element, wait=0, strip=False):
        sleep(wait)
        self._temp_element = element
        if self._web_element_type not in str(type(self._temp_element)):
            self._temp_element = self.check_all_options(element)
        try:
            if self._temp_element is not None:
                allure.attach(self._temp_element.text, 'Text', allure.attachment_type.TEXT)
                if strip:
                    return re.sub(r"\s+", "", self._temp_element.text)
                return self._temp_element.text
        except Exception as e:
            assert False, f"Failed to get text to element: {element}"

    @allure.step
    def check_all_options(self, element):
        self._temp_element = self.find_element_by_xpath(element)
        if self._temp_element is not None:
            return self._temp_element

        # check id option
        self._temp_element = self.find_element_by_id(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_css(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_link_text(element)
        if self._temp_element is not None:
            return self._temp_element

        # check name option
        self._temp_element = self.find_element_by_name(element)
        if self._temp_element is not None:
            return self._temp_element

        return

    @allure.step
    def find_element_by_id(self, element):
        try:
            self._temp_element = self.driver.find_element_by_id(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def find_element_by_class_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_class_name(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def find_element_by_xpath(self, element):
        try:
            self._temp_element = self.driver.find_element_by_xpath(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def find_element_by_name(self, element):
        try:
            self._temp_element = self.driver.find_element_by_name(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def find_element_by_css(self, element):
        try:
            self._temp_element = self.driver.find_element_by_css(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def find_element_by_link_text(self, element):
        try:
            self._temp_element = self.driver.find_element_by_link_text(element)
            print(f"Found element: {element}")
            return self._temp_element
        except Exception:
            print(f"Failed to find element: {element}")
        return False

    @allure.step
    def get_page_source(self):
        try:
            source = self.driver.page_source
            allure.attach(source, 'Page source', allure.attachment_type.TEXT)
            return source
        except Exception:
            return False

    @allure.step
    def get_body(self):
        try:
            allure.attach(self.driver.find_element_by_tag_name("body").text, 'Body',
                          allure.attachment_type.TEXT)
            return self.driver.find_element_by_tag_name("body").text
        except Exception:
            return False

    @allure.step
    def get_href(self, element):
        try:
            if element is not None:
                self._temp_element = element.get_attribute("href")
                allure.attach(self._temp_element, 'href', allure.attachment_type.TEXT)
                return self._temp_element

        except Exception:
            print(f"Failed to find element: {element}")
        return False



import allure

from bqpackage.selenium_base.selenium_actions import SeleniumActions
from bqpackage.selenium_base.selenium_config import SeleniumConfig
from bqpackage.shared import run_parallel as parallel


# Test 1 - check the default parameters
@allure.feature('Fixtures')
def test_1():
    selenium = SeleniumConfig()
    selenium.init_driver()
    selenium.tear_down()


# # Test 2 - check the headless option
@allure.feature('Fixtures')
def test_2():
    selenium = SeleniumConfig()
    selenium.init_driver(headless=True)
    selenium.tear_down()


# # test 3 - check the url option
@allure.feature('Fixtures')
def test_3():
    selenium = SeleniumConfig()
    selenium.init_driver()
    selenium.get_url('"https://www.google.com/"')
    selenium.tear_down()


# test 4 check the proxy server
@allure.feature('Fixtures')
def test_4():
    selenium = SeleniumConfig()
    selenium.init_driver(proxy=True)
    selenium.get_url('"https://www.google.com/"')
    selenium.tear_down()


# test 5 - check the parallel function
@allure.feature('Fixtures')
def test_5():
    # TODO
    pass